package org.artifactory.repo.cache.expirable;

import lombok.extern.slf4j.Slf4j;

import java.nio.file.InvalidPathException;
import java.nio.file.Paths;

@Slf4j
public class InternalMetadataChecker {

    public static final String INTERNAL_METADATA_FOLDER = ".jfrog/";

    /**
     * Indicates whether the specified path is internal metadata path
     * and should be hidden from standard outputs like UI or AQL
     *
     * @param path the path to check
     * @return true if the path is metadata, false otherwise
     */
    public static boolean isInternalMetadata(String path) {
        try {
            return Paths.get(path).startsWith(INTERNAL_METADATA_FOLDER);
        } catch (InvalidPathException e) {
            log.error("Path {} couldn't be parsed.", path);
            log.debug(e.getMessage(), e);
        }
        return false;
    }
}
