/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.repo.service.flexible.context;

import com.google.common.collect.Lists;
import lombok.NonNull;
import org.artifactory.md.Properties;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.service.flexible.MoveCopyItemInfo;

import java.util.List;
import java.util.function.Function;

/**
 * @author gidis
 */
public class MoveCopyContext {

    private final RepoPath sourceRepoPath;
    private final String targetKey;
    private final String targetPath;
    private Function<RepoPath, Properties> addProps; // get props for TARGET RepoPath
    private Function<RepoPath, List<String>> removeProps; // get list of property keys to delete from TARGET RepoPath
    private boolean skipProperties;
    private boolean dryRun;
    private boolean failFast;
    private int transactionSize;
    private boolean copy;
    private boolean copyContent;
    private boolean suppressLayouts;
    private final List<MoveCopyItemInfo> foldersToDelete = Lists.newArrayList();

    public MoveCopyContext(RepoPath sourceRepoPath, String targetKey, String targetPath) {
        this.sourceRepoPath = sourceRepoPath;
        this.targetKey = targetKey;
        this.targetPath = targetPath;
    }

    public RepoPath getSourceRepoPath() {
        return sourceRepoPath;
    }

    public String getTargetKey() {
        return targetKey;
    }

    public String getTargetPath() {
        return targetPath;
    }

    @NonNull
    public Function<RepoPath, Properties> getAddProps() {
        return addProps;
    }

    @NonNull
    public Function<RepoPath, List<String>> getRemoveProps() {
        return removeProps;
    }

    public boolean isSkipProperties() {
        return skipProperties;
    }

    public boolean isDryRun() {
        return dryRun;
    }

    public boolean isFailFast() {
        return failFast;
    }

    public int getTransactionSize() {
        return transactionSize;
    }

    public boolean isCopy() {
        return copy;
    }

    public boolean isCopyContent() {
        return copyContent;
    }

    public boolean isSuppressLayouts() {
        return suppressLayouts;
    }

    public MoveCopyContext setCopy(boolean copy) {
        this.copy = copy;
        return this;
    }

    public MoveCopyContext setFailFast(boolean failFast) {
        this.failFast = failFast;
        return this;
    }

    public MoveCopyContext setDryRun(boolean dryRun) {
        this.dryRun = dryRun;
        return this;
    }

    public MoveCopyContext setSkipProperties(boolean skipProperties) {
        this.skipProperties = skipProperties;
        return this;
    }

    public MoveCopyContext setRemoveProperties(@NonNull Function<RepoPath, List<String>> removeProperties) {
        this.removeProps = removeProperties;
        return this;
    }

    public MoveCopyContext setAddProperties(@NonNull Function<RepoPath, Properties> addProperties) {
        this.addProps = addProperties;
        return this;
    }

    public MoveCopyContext setCopyContent(boolean copyContent) {
        this.copyContent = copyContent;
        return this;
    }

    public MoveCopyContext setSuppressLayouts(boolean suppressLayouts) {
        this.suppressLayouts = suppressLayouts;
        return this;
    }

    public MoveCopyContext setTransactionSize(int transactionSize) {
        this.transactionSize = transactionSize;
        return this;
    }

    public List<MoveCopyItemInfo> getFoldersToDelete() {
        return foldersToDelete;
    }
}
