package org.artifactory.repo.service.validation;

import org.artifactory.api.security.AuthorizationException;
import org.artifactory.model.RepoConfig;

import java.util.Collection;

/**
 * @author Mor Merhav
 */
public interface ProjectAdminRepoConfigValidator {

    /**
     * Validate whether project admin is able to perform changes on the input repo config.
     * Super admin is always cleared to perform all changes.
     * Exists silently if {@code repoConfig} is null.
     *
     * @param repoConfig the repo config to validate for project admin authorization
     * @throws AuthorizationException if the current user is not a super admin nor a project admin, or a project
     * admin attempted to perform an illegal operation
     */
    void validate(RepoConfig repoConfig);

    /**
     * Validate whether project admin is able to perform changes on the input repo configs
     * Super admin is always cleared to perform all changes.
     * Exists silently if {@code repoConfigs} is null or empty.
     *
     * @param repoConfigs the repo configs to validate for project admin authorization
     * @throws AuthorizationException if the current user is not a super admin nor a project admin, or a project
     * admin attempted to perform an illegal operation
     */
    void validate(Collection<? extends RepoConfig> repoConfigs);

}
