package org.artifactory.repo.service.validation;

import org.artifactory.api.security.AuthorizationException;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.model.RepoConfig;
import org.artifactory.repo.RepoDetailsType;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import java.util.Collection;
import java.util.Collections;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Mor Merhav
 */
class ProjectAdminRepoConfigValidatorImplTest {

    @Mock
    private AuthorizationService authService;
    @InjectMocks
    private ProjectAdminRepoConfigValidatorImpl projectAdminRepoConfigValidator;

    private AutoCloseable mockitoAutoCloseable;

    @BeforeEach
    void setUp() {
        mockitoAutoCloseable = MockitoAnnotations.openMocks(this);
    }

    @AfterEach
    void tearDown() throws Exception {
        mockitoAutoCloseable.close();
    }

    @Test
    void nullRepoConfigValidationPasses() {
        projectAdminRepoConfigValidator.validate((RepoConfig) null);
        projectAdminRepoConfigValidator.validate((Collection<? extends RepoConfig>) null);
    }

    @Test
    void emptyRepoConfigValidationPasses() {
        when(authService.isAnyProjectAdmin()).thenReturn(true);
        projectAdminRepoConfigValidator.validate(Collections.emptySet());
    }

    @Test
    void userNotProjectAdminThrowsException() {
        when(authService.isAnyProjectAdmin()).thenReturn(false);
        Set<RepoConfig> repoConfigs = Set.of(mock(RepoConfig.class));
        assertThrows(AuthorizationException.class,
                () -> projectAdminRepoConfigValidator.validate(repoConfigs),
                "Only an admin user can save the artifactory configuration.");
    }

    @Test
    void superAdminIsAlwaysCleared() {
        when(authService.isAnyProjectAdmin()).thenReturn(true);
        when(authService.isAdmin()).thenReturn(true);
        RepoConfig repoConfig = mock(RepoConfig.class);
        when(repoConfig.getRepoType()).thenReturn(RepoDetailsType.FEDERATED);
        Set<RepoConfig> repoConfigs = Set.of(repoConfig);

        projectAdminRepoConfigValidator.validate(repoConfigs);
    }
}