package org.artifactory.repo.service.validation;

import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

class RepositoryNameYamlConfigurationValidatorTest {

    @Mock
    private CentralConfigDescriptor centralConfigDescriptor;
    private AutoCloseable mockitoAutoCloseable;

    @BeforeEach
    void setUp() {
        mockitoAutoCloseable = MockitoAnnotations.openMocks(this);
    }

    @AfterEach
    void tearDown() throws Exception {
        mockitoAutoCloseable.close();
    }

    @Test
    void testValidRepoName() {
        when(centralConfigDescriptor.isKeyAvailable(anyString())).thenReturn(true);
        RepositoryNameYamlConfigurationValidator.validateRepoName("valid-repo-name", "remote", centralConfigDescriptor);
    }

    @ParameterizedTest
    @MethodSource("repoNames")
    void testInvalidRepoName(String repoName) {
        when(centralConfigDescriptor.isKeyAvailable(anyString())).thenReturn(true);
        assertThrows(IllegalArgumentException.class,
                () -> RepositoryNameYamlConfigurationValidator.validateRepoName(repoName, "local",
                        centralConfigDescriptor));
    }

    @Test
    void testKeyUnavailableInCentralConfigDescriptor() {
        when(centralConfigDescriptor.isKeyAvailable(anyString())).thenReturn(false);
        assertThrows(IllegalArgumentException.class,
                () -> RepositoryNameYamlConfigurationValidator.validateRepoName("valid-repo-name", "remote",
                        centralConfigDescriptor));
    }

    private static Object[][] repoNames() {
        return new Object[][] {
                {"jfrog-artifactory-system-repo-name"},
                {"jfrog-system-reserved-repo-name"},
                {"some-repo-?-name"},
                {"/some-repo-name"},
                {"some-repo-name*"},
                {""},
                {null},
                {"very-very-l0000000oooooooooooooooooooooooooooooooooooong-local-repo"},
                {"_intransit"},
                {"artifactory-pipe-info"}
        };
    }

}
