package org.artifactory.common.profile;

import lombok.NonNull;

import java.util.Map;

/**
 * Contains profiles names for gradual rollout of legacy packages implementations.
 * Enum is not applicable here because values are used in annotation @Profile. It requires constants and enums aren't
 * comply those requirements.
 *
 * @author Volodymyr Shulga
 */
public interface PackageProfile {
    String NEW_NUGET_IMPL = "nugetNew";
    String NEW_NPM_IMPL = "npmNew";

    Map<String, String> values = Map.of(
            toPackageType(NEW_NPM_IMPL), NEW_NPM_IMPL,
            toPackageType(NEW_NUGET_IMPL), NEW_NUGET_IMPL);

    static String toPackageProfile(@NonNull String packageType) {
        String ret = values.get(packageType.toLowerCase());
        if (ret != null) {
            return ret;
        } else {
            throw new IllegalArgumentException(String.format("PackageProfile for %s not found", packageType));
        }
    }

    static String toPackageType(@NonNull String profileName) {
        return profileName.replace("New", "");
    }
}
