/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.sapi.interceptor.context;

import lombok.Getter;
import org.artifactory.repo.RepoPath;

/**
 * A context to pass when performing item deletion.
 *
 * @author Yossi Shaul
 */
@Getter
public class DeleteContext extends AbstractInterceptorContext {

    private final RepoPath repoPath;
    private boolean triggerMetadataCalculation;
    private boolean avoidBuildDeleteInterceptor;
    private boolean allowAsyncDelete;
    private boolean skipTrashcan;

    public DeleteContext(RepoPath repoPath) {
        this.repoPath = repoPath;
    }

    public DeleteContext triggerMetadataCalculation(boolean triggerMetadataCalculation) {
        this.triggerMetadataCalculation = triggerMetadataCalculation;
        return this;
    }

    public DeleteContext triggeredByMove() {
        setTriggeredByMove(true);
        return this;
    }

    public DeleteContext avoidBuildDeleteInterceptor() {
        avoidBuildDeleteInterceptor = true;
        return this;
    }

    public DeleteContext allowAsyncDelete(boolean allowAsyncDelete) {
        this.allowAsyncDelete = allowAsyncDelete;
        return this;
    }

    public DeleteContext skipTrashcan(boolean skipTrashcan) {
        this.skipTrashcan = skipTrashcan;
        return this;
    }
}
