package org.artifactory.concurrent;

import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.common.home.ArtifactoryHome;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;

/**
 * @author Uriah Levy
 */
@Test
public class ArtifactoryCallableTest {

    @Mock
    ArtifactoryContext artifactoryContext;

    @Mock
    ArtifactoryHome artifactoryHome;

    @BeforeClass
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        when(artifactoryContext.getArtifactoryHome()).thenReturn(artifactoryHome);
    }

    @Test(expectedExceptions = IllegalStateException.class, expectedExceptionsMessageRegExp = ".*Artifactory home.*")
    public void testArtifactoryCallable() throws Exception {
        String result = new ArtifactoryCallable<>(() -> {
            // Context is bound
            assertNotNull(ContextHelper.get());
            assertNotNull(ArtifactoryHome.get());
            return "foo";
        }, artifactoryContext, null).call();

        assertEquals(result, "foo");
        assertNull(ContextHelper.get());
        // throws
        ArtifactoryHome.get();
    }
}