package org.artifactory.config;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.descriptor.security.EncryptionPolicy;

import java.util.HashMap;
import java.util.Map;

@Slf4j
public final class AccessConfigHolder {

    public static final String SECURITY_MAP = "security";
    public static final String AUTHENTICATION_MAP = "authentication";
    public static final String AUTHENTICATION_PASSWORD_ENCRYPTION = "password-encryption";
    public static final String AUTHENTICATION_DISABLE_REMEMBER_ME = "disable-remember-me";
    public static final String AUTHENTICATION_BASIC_AUTH_ENABLED = "basic-authentication-enabled";
    public static final String SECURITY_USER_LOCK_POLICY_MAP = "user-lock-policy";
    public static final String SECURITY_ANONYMOUS_ACCESS_ENABLED = "anonymous-access-enabled";
    public static final String USER_LOCK_POLICY_ATTEMPTS = "attempts";
    public static final String USER_LOCK_POLICY_PASSWORD_EXPIRY_DAYS = "password-expiry-days";
    private final Map<String, Object> configMap = new HashMap<>();
    private final Object lockObject = new Object();

    private AccessConfigHolder() {
    }

    public static AccessConfigHolder getInstance() {
        return InstanceHolder.INSTANCE;
    }

    public Map<String, Object> drainConfig() {
        synchronized (lockObject) {
            Map<String, Object> mapCopy = new HashMap<>(configMap);
            configMap.clear();
            return mapCopy;
        }
    }

    public void addPasswordEncryptionPolicy(EncryptionPolicy encryptionPolicy) {
        AccessConfigMapUtils.addPasswordEncryptionPolicy(encryptionPolicy, configMap);
    }

    public void addDisableRememberMe(boolean value) {
        addToAuthentication(AUTHENTICATION_DISABLE_REMEMBER_ME, value);
    }

    public void addBasicAuthenticationEnabled(boolean value) {
        addToAuthentication(AUTHENTICATION_BASIC_AUTH_ENABLED, value);
    }

    public void addLoginAttempts(int loginAttempts) {
        addToUserLockPolicy(USER_LOCK_POLICY_ATTEMPTS, loginAttempts);
    }

    public boolean isEmpty() {
        return configMap.isEmpty();
    }

    public void addPasswordExpiryDays(int passwordExpiryDays) {
        addToUserLockPolicy(USER_LOCK_POLICY_PASSWORD_EXPIRY_DAYS, passwordExpiryDays);
    }

    public void addAnonAccessEnabled(Boolean anonAccessEnabled) {
        synchronized (lockObject) {
            Map<String, Object> securityMap = AccessConfigMapUtils.getOrCreate(SECURITY_MAP, configMap);
            securityMap.put(SECURITY_ANONYMOUS_ACCESS_ENABLED, anonAccessEnabled);
        }
    }

    private void addToAuthentication(String key, Object value) {
        synchronized (lockObject) {
            AccessConfigMapUtils.addToAuthentication(key, value, configMap);
        }
    }

    private void addToUserLockPolicy(String key, Integer value) {
        synchronized (lockObject) {
            AccessConfigMapUtils.addToUserLockPolicy(key, value, configMap);
        }
    }

    private static final class InstanceHolder {
        private static final AccessConfigHolder INSTANCE = new AccessConfigHolder();
    }
}