package org.artifactory.descriptor.repo;

import lombok.Getter;
import lombok.Setter;
import org.artifactory.descriptor.Descriptor;
import org.jfrog.common.config.diff.DiffKey;
import org.jfrog.common.config.diff.GenerateDiffFunction;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlType;
import java.util.Objects;

@XmlType(name = "FederatedMembersType", propOrder = {"url", "mode", "enabled"},
        namespace = Descriptor.NS)
@GenerateDiffFunction
@Getter
@Setter
public class FederatedMember implements Descriptor {

    @XmlID
    @XmlElement(required = true)
    @DiffKey
    private String url;
    private String mode;

    @XmlElement(defaultValue = "true", required = true)
    @DiffKey
    private boolean enabled = true;

    @Override
    public int hashCode() {
        return Objects.hash(url, enabled);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        FederatedMember that = (FederatedMember) o;
        if (enabled != that.enabled) {
            return false;
        }

        return Objects.equals(url, that.url);
    }

    @Override
    public String toString() {
        return "FederatedMember{" +
                "url='" + url + '\'' +
                ", enabled=" + enabled +
                '}';
    }
}
