package org.artifactory.repo;

import org.artifactory.model.RepoConfig;
import org.jfrog.common.JsonUtils;

import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Tamar Rosen
 */
public class RepositoriesConverterManager {
    Set<RepositoriesConfigSubConverter> subConverterSet;

    public RepositoriesConverterManager(Set<RepositoriesConfigSubConverter> subConverterSet) {
        this.subConverterSet = subConverterSet;
    }

    /**
     * Gets a byte array representing a {@link RepoConfig} implementation
     */
    public RepoConfig convert(byte[] toConvert) {
        byte[] bytes = convertAsBytes(toConvert);
        return JsonUtils.getInstance().readValue(bytes, RepoConfig.class);
    }

    public byte[] convertAsBytes(byte[] toConvert) {
        Map<String, Object> repositoryConfigMap = JsonUtils.getInstance().readValue(toConvert, Map.class);
        // TODO[t.r] - support flat RepoConfig properties
        Map<String, Map<String, Object>> filteredRepoConfigMap = repositoryConfigMap.entrySet().stream()
                .filter(entry -> entry.getValue() instanceof Map)
                .collect(Collectors.toMap(Map.Entry::getKey, e -> (Map<String, Object>) e.getValue()));

        filteredRepoConfigMap.replaceAll(this::convert);
        return JsonUtils.getInstance().valueToByteArray(repositoryConfigMap);
    }

    private Map<String, Object> convert(String key, Map<String, Object> subConfig) {
        Optional<RepositoriesConfigSubConverter> interestedConverter = subConverterSet.stream()
                .filter(subConverter -> subConverter.isInterested(key))
                .findFirst();
        if (interestedConverter.isPresent()) {
            return interestedConverter.get().convert(subConfig);
        }
        return subConfig;
    }
}
