package org.artifactory.repo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

import static org.artifactory.repo.PackageTypeConfigConverter.PACKAGE_TYPE_CONFIG_KEY;
import static org.artifactory.repo.RepositoryConfigConverterEnum.getRelevantConverters;

/**
 * @author Tamar Rosen
 */
@Component
public class RepositoryConfigBaseConverter implements RepositoriesConfigSubConverter {
    private static final Logger log = LoggerFactory.getLogger(RepositoryConfigBaseConverter.class);
    private static final int VERSION = 1;

    @Override
    public boolean isInterested(String key) {
        return !PACKAGE_TYPE_CONFIG_KEY.equals(key);
    }

    @Override
    public Map<String, Object> convert(Map<String, Object> toConvert) {
        if (!toConvert.containsKey("modelVersion")) {
            return toConvert;
        }
        int modelVersion = (int) toConvert.get("modelVersion");
        List<RepositoriesConverter> converters = getRelevantConverters(modelVersion);
        if (converters.isEmpty()) {
            log.debug("No Repository Config converters found between version {} and {}", modelVersion, VERSION);
        } else {
            log.info("Starting Repository Config conversion from {} to {}", modelVersion, VERSION);
            for (RepositoriesConverter converter : converters) {
                if (converter.isInterested(modelVersion)) {
                    converter.convert(toConvert);
                } else {
                    log.info("Not running Repository Config converter '{}' (converter criteria not met)",
                            converter.getClass().getSimpleName());
                }
            }
            log.info("Finished Repository Config conversion from {} to {}", modelVersion, VERSION);
        }
        return toConvert;
    }
}
