package org.artifactory.descriptor.repo;

import com.google.common.collect.Maps;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.property.PropertySet;
import org.artifactory.model.FederatedRepoConfig;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.common.JsonUtils;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.util.ReflectionUtils;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Map;

import static com.google.gson.JsonParser.parseString;
import static org.artifactory.descriptor.repo.RandomObjectPopulator.createAndFill;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * @author Tamar Rosen
 */
public class FederatedRepoDescriptorTest extends ArtifactoryHomeBoundTest {
    @Mock
    private CentralConfigDescriptor centralConfigDescriptor;

    @BeforeMethod
    public void setUp() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    public void testDescriptorToAndFromRepositoryConfig() throws Exception {
        // given
        FederatedRepoDescriptor sourceFedRepoDescriptor = createAndFill(FederatedRepoDescriptor.class);

        // Generating all types of package configs
        Map<String, String> packageTypeConfig = Maps.newHashMap();
        for (RepoType value : RepoType.values()) {
            sourceFedRepoDescriptor.buildPackageTypeConfig(value, packageTypeConfig);
        }

        RepoLayout repoLayout = new RepoLayout();
        repoLayout.setName(sourceFedRepoDescriptor.getRepoLayout().getName());
        KeyPairDescriptor primaryKeyPair = new KeyPairDescriptor();
        primaryKeyPair.setKey(sourceFedRepoDescriptor.getPrimaryKeyPairDescriptor().getKey());
        KeyPairDescriptor secondaryKeyPair = new KeyPairDescriptor();
        secondaryKeyPair.setKey(sourceFedRepoDescriptor.getSecondaryKeyPairDescriptor().getKey());

        PropertySet propertySet1 = new PropertySet();
        propertySet1.setName("propertySetRef1");
        PropertySet propertySet2 = new PropertySet();
        propertySet2.setName("propertySetRef2");
        ProxyDescriptor proxyDescriptor = new ProxyDescriptor();
        proxyDescriptor.setKey(sourceFedRepoDescriptor.getProxy().getKey());

        when(centralConfigDescriptor.getRepoLayouts()).thenReturn(List.of(repoLayout));
        when(centralConfigDescriptor.getPropertySets()).thenReturn(List.of(propertySet1, propertySet2));
        when(centralConfigDescriptor.getKeyPairs()).thenReturn(List.of(primaryKeyPair, secondaryKeyPair));
        when(centralConfigDescriptor.getProxies()).thenReturn(List.of(proxyDescriptor));

        // when
        FederatedRepoConfig sourceRepositoryConfig = sourceFedRepoDescriptor.toRepositoryConfig();
        Field packageTypeConfigField = ReflectionUtils.findField(FederatedRepoConfig.class, "packageTypeConfig");
        Assert.assertNotNull(packageTypeConfigField);
        ReflectionUtils.makeAccessible(packageTypeConfigField);
        ReflectionUtils.setField(packageTypeConfigField, sourceRepositoryConfig, packageTypeConfig);

        FederatedRepoDescriptor mappedLocalDescriptor =
                FederatedRepoDescriptor.fromRepositoryConfig(centralConfigDescriptor, sourceRepositoryConfig);

        // then
        for (RepoType value : RepoType.values()) {
            mappedLocalDescriptor.setPackageTypeConfig(sourceRepositoryConfig.getPackageTypeConfig(), value);
        }
        String sourceFedRepoDescriptorJson = JsonUtils.getInstance().writeValueAsString(sourceFedRepoDescriptor);
        String mappedFedRepoDescriptorJson = JsonUtils.getInstance().writeValueAsString(mappedLocalDescriptor);

        System.out.println(sourceFedRepoDescriptorJson);
        System.out.println(mappedFedRepoDescriptorJson);

        assertEquals(parseString(mappedFedRepoDescriptorJson), parseString(sourceFedRepoDescriptorJson));
    }
}