package org.artifactory.core.api.ph.service;

import org.apache.http.client.methods.CloseableHttpResponse;

import javax.annotation.Nullable;

/**
 * API that Artifactory provides for the package handler layer to interact with Xray
 */
public interface PackageXrayService {

    /**
     * @return True if Xray is configured in descriptor and enabled
     */
    boolean isXrayEnabled();

    /**
     * @return True if Xray client reached Xray server and is in stable mode
     */
    default boolean isXrayAliveByHeartbeat() {
        return false;
    }

    /**
     * return npm audit from xray
     *
     * @param strategy                - full/quick/bulk
     * @param auditRequestData        - client request data
     * @param optionalReportToEnhance - null or audit report from npm registry
     * @return enhanced audit report or null if failed
     */
    @Nullable
    CloseableHttpResponse npmAuditReport(String strategy, String auditRequestData, String optionalReportToEnhance);
}
