/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.db.event.dao;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.storage.db.event.dao.query.MirrorsLagQueryHandler;
import org.artifactory.storage.db.event.dao.query.QueueEventsTypesQueryHandler;
import org.artifactory.storage.db.util.BaseDao;
import org.artifactory.storage.db.util.JdbcHelper;
import org.artifactory.storage.event.EventsTypesStatusInfo;
import org.artifactory.storage.event.cursor.RawMirrorLagInfo;
import org.jfrog.storage.DbProperties;
import org.jfrog.storage.DbType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import javax.annotation.PostConstruct;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;


@Slf4j
@Repository
public class NodeEventsAndNodeEventCursorDao extends BaseDao {

    private DbProperties dbProperties;
    private DbType dbType;

    private MirrorsLagQueryHandler mirrorsLagQueryHandler;
    private QueueEventsTypesQueryHandler queueEventsTypesQueryHandler;

    @Autowired
    public NodeEventsAndNodeEventCursorDao(JdbcHelper jdbcHelper) {
        super(jdbcHelper);
    }

    public DbType getDbType() {
        return dbType;
    }

    @Autowired
    public void setDbProperties(DbProperties dbProperties) {
        this.dbProperties = dbProperties;
    }

    @PostConstruct
    public void init() {
        dbType = dbProperties.getDbType();
        this.mirrorsLagQueryHandler = new MirrorsLagQueryHandler(dbType);
        this.queueEventsTypesQueryHandler = new QueueEventsTypesQueryHandler(dbType);
    }


    public List<RawMirrorLagInfo> getMirrorsLag() throws SQLException {
        List<RawMirrorLagInfo> mirrorsWithLag = new ArrayList<>();
        List<RawMirrorLagInfo> mirrorsWithoutLag = new ArrayList<>();
        String sqlQuery = this.mirrorsLagQueryHandler.getQueryByDBType();

        try (ResultSet resultSet = jdbcHelper.executeSelect(sqlQuery)) {
            long currentTimeStamp = System.currentTimeMillis();
            while (resultSet.next()) {
                mirrorsLagQueryHandler.createMirrorLagFromResultSet(resultSet, currentTimeStamp,
                        mirrorsWithLag, mirrorsWithoutLag);
            }
        }
        mirrorsWithLag.addAll(mirrorsWithoutLag);
        return mirrorsWithLag;
    }

    private void EventTypesInfoFromResultSet(ResultSet rs, long currentTimeStamp,
            List<EventsTypesStatusInfo> withLagCursors,
            List<EventsTypesStatusInfo> noLagCursors) throws SQLException {
        long timeStampForNextEvent = rs.getLong("tsnextevent");
        long lagInMS = currentTimeStamp - timeStampForNextEvent;
        // if there is no next event, the returned value of tsNextEvent will be 0, in order to fill thuse tsNextEvent
        // at the end of the list we split them and then join them at the end of the list.
        // also to ignore events that after current timestamp
        if (timeStampForNextEvent == 0 || lagInMS < 0) {
            noLagCursors.add(
                    new EventsTypesStatusInfo(rs.getString("operator_id"), rs.getString("status"), rs.getLong("type_1"),
                            rs.getLong("type_2"), rs.getLong("type_3"), rs.getLong("type_4"), 0, 0));
        } else {
            withLagCursors.add(
                    new EventsTypesStatusInfo(rs.getString("operator_id"), rs.getString("status"), rs.getLong("type_1"),
                            rs.getLong("type_2"), rs.getLong("type_3"), rs.getLong("type_4"), lagInMS,
                            timeStampForNextEvent));
        }
    }

    public List<EventsTypesStatusInfo> getEventsTypesStatus(String repoKey, long currentTime) throws SQLException {
        String sqlQuery;
        // if there is no next event, the returned value of tsNextEvent will be 0, in order to fill thuse tsNextEvent
        // at the end of the list we split them and then join them at the end of the list.
        List<EventsTypesStatusInfo> mirrorsWithLag = new ArrayList<>();
        List<EventsTypesStatusInfo> mirrorsWithoutLag = new ArrayList<>();
        sqlQuery = this.queueEventsTypesQueryHandler.getQueryByDBType();
        try (ResultSet rs = jdbcHelper.executeSelect(sqlQuery,
                this.queueEventsTypesQueryHandler.fillStatementParams(repoKey, currentTime))) {
            while (rs.next()) {
                EventTypesInfoFromResultSet(rs, currentTime, mirrorsWithLag, mirrorsWithoutLag);
            }
        }
        mirrorsWithLag.addAll(mirrorsWithoutLag);
        return mirrorsWithLag;
    }
}
