package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.converter.DbVersion;
import org.jfrog.converter.model.ChangeLogContext;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v118;
import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v119;


@Test
public class V118toV119CursorEventTableUpgradeTest extends UpgradeBaseTest {

    private long beforeTime = System.currentTimeMillis();

    @BeforeMethod
    public void setup() throws IOException, SQLException {
        resetToVersion(ArtifactoryDBVersion.v117);
        importSql("/sql/event_cursor_status_and_updated.sql");
        final List<ArtifactoryDBVersion> dbVersions = List.of(v118, v119);
        for (DbVersion dbVersion : dbVersions) {
            try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                    dbProperties.getDbType(), dbVersion)) {
                dbVersion.converter().convert(context);
            }
        }
    }

    public void testAddStatusColumnToEventCursorTable() throws SQLException {
        String tbName = "node_event_cursor";
        Assert.assertTrue(tableExists(tbName));
        Assert.assertTrue(columnExists(tbName, "status"));
        Assert.assertTrue(columnExists(tbName, "last_updated"));

        long afterTime = System.currentTimeMillis();

        try (ResultSet res = jdbcHelper.executeSelect("SELECT * FROM node_event_cursor")){
            Assert.assertTrue(res.next());
            Assert.assertEquals(res.getString("status"), "SYNC");
            long columnLastUpdated = res.getLong("last_updated");
            Assert.assertTrue(columnLastUpdated <= afterTime);
        }
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return v119;
    }
}
