package org.artifactory.ui.rest.resource.artifacts.browse.treebrowser.tabs.views;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.common.profile.PackageProfile;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.ph.PackageResourceService;
import org.artifactory.ui.rest.model.artifacts.browse.treebrowser.tabs.nugetinfo.NugetArtifactInfo;
import org.artifactory.ui.rest.service.artifacts.browse.BrowseServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import java.util.Arrays;

/**
 * @author Volodymyr Shulga
 */
@RolesAllowed({AuthorizationService.ROLE_ADMIN, AuthorizationService.ROLE_USER})
@Component
@Path("views/nuget")
public class ViewNugetResource {

    private BrowseServiceFactory browseFactory;
    private ViewsResource viewsResource;
    private final boolean legacyProfile;
    private final PackageResourceService packageResourceService;

    @Autowired
    public ViewNugetResource(BrowseServiceFactory browseServiceFactory, ViewsResource viewsResource,
            ConfigurableEnvironment env, PackageResourceService packageResourceService) {
        this.viewsResource = viewsResource;
        this.browseFactory = browseServiceFactory;
        this.legacyProfile = !Arrays.asList(env.getActiveProfiles()).contains(PackageProfile.NEW_NUGET_IMPL);
        this.packageResourceService = packageResourceService;
    }

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    @Consumes(MediaType.APPLICATION_JSON)
    public Response viewNuget(NugetArtifactInfo nugetArtifactInfo, @Context HttpServletRequest servletRequest,
            @Context HttpServletResponse servletResponse, @Context  Request request, @Context UriInfo uriInfo,
            @Context HttpHeaders httpHeaders) throws PackageException {
        if (legacyProfile) {
            return viewsResource.runService(browseFactory.nugetViewService(), nugetArtifactInfo, servletRequest,
                    servletResponse, request, uriInfo, httpHeaders);
        } else {
            return packageResourceService.getPackageInfo(nugetArtifactInfo.getRepoKey(), nugetArtifactInfo.getPath());
        }
    }

}
