package org.artifactory.ui.rest.resource.artifacts.versions;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.common.profile.PackageProfile;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.ph.PackageResourceService;
import org.artifactory.ui.rest.service.artifacts.search.NativeSearchServiceFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.stereotype.Component;

import javax.annotation.security.RolesAllowed;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import java.util.Arrays;
import java.util.List;

import static org.artifactory.ui.rest.service.artifacts.search.packagesearch.util.PackageNativeRestConstants.PATH;

/**
 * @author Asaf Zalcman
 */
@RolesAllowed({AuthorizationService.ROLE_ADMIN, AuthorizationService.ROLE_USER})
@Component
@Path("v1/native/versions/{type}/readme")
public class VersionsReadmeResource {

    private final NativeSearchServiceFactory nativeSearchServiceFactory;

    private final VersionNativeResource versionNativeResource;
    private final boolean legacyProfile;
    private final PackageResourceService packageResourceService;

    @Autowired
    public VersionsReadmeResource(
            NativeSearchServiceFactory nativeSearchServiceFactory, VersionNativeResource versionNativeResource,
            ConfigurableEnvironment env, PackageResourceService packageResourceService) {
        this.nativeSearchServiceFactory = nativeSearchServiceFactory;
        this.versionNativeResource = versionNativeResource;
        this.legacyProfile = !Arrays.asList(env.getActiveProfiles()).contains(PackageProfile.NEW_NPM_IMPL);
        this.packageResourceService = packageResourceService;
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response getReadme(
            @Context HttpServletRequest servletRequest,
            @Context HttpServletResponse servletResponse, @Context Request request, @Context UriInfo uriInfo,
            @Context HttpHeaders httpHeaders) throws PackageException {
        if (legacyProfile) {
            return versionNativeResource.runService(nativeSearchServiceFactory.versionNativeReadmeService(),
                    servletRequest,
                    servletResponse, request, uriInfo, httpHeaders);
        } else {
            String path = "";
            List<String> valueList = uriInfo.getQueryParameters().get(PATH);
            if (valueList != null && !valueList.isEmpty()) {
                path = valueList.get(0);
            }
            return packageResourceService.getVersionNativeReadme(path);
        }
    }

}
