package org.artifactory.ui.rest.model.artifacts.browse.treebrowser.action.recalculateindex;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.pypi.PypiAddon;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.model.FederatedRepoConfig;
import org.mockito.Answers;
import org.mockito.Mockito;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class PypiIndexCalculatorTest {

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testCalculateIndex() throws Exception {
        ArtifactoryContext context = mock(ArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        AddonsManager addonsManager = mock(AddonsManager.class);
        PypiAddon pypiAddon = mock(PypiAddon.class);
        RepositoryService repositoryService = mock(RepositoryService.class);

        when(addonsManager.addonByType(PypiAddon.class)).thenReturn(pypiAddon);
        when(context.beanForType(AddonsManager.class)).thenReturn(addonsManager);
        when(context.beanForType(RepositoryService.class)).thenReturn(repositoryService);
        CentralConfigDescriptor descriptor = mock(CentralConfigDescriptor.class);
        String repoName = "repo";
        FederatedRepoConfig federatedRepoConfig = Mockito.mock(FederatedRepoConfig.class, Answers.RETURNS_DEEP_STUBS);
        when(repositoryService.localOrFederatedRepoConfigByKey(repoName)).thenReturn(federatedRepoConfig);
        when(federatedRepoConfig.getKey()).thenReturn(repoName);
        CentralConfigService service = mock(CentralConfigService.class);
        when(service.getDescriptor()).thenReturn(descriptor);
        when(context.getCentralConfig()).thenReturn(service);
        PypiIndexCalculator indexCalculator = new PypiIndexCalculator();
        indexCalculator.setRepoKey(repoName);
        indexCalculator.calculateIndex();
        verify(pypiAddon, times(1)).reindex(repoName, true);
    }
}