package org.artifactory.ui.rest.resource.artifacts.versions;

import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.ph.PackageResourceService;
import org.artifactory.ui.rest.service.artifacts.search.NativeSearchServiceFactory;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.core.env.ConfigurableEnvironment;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.Request;
import javax.ws.rs.core.UriInfo;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;


/**
 * @author Asaf Zalcman
 */
class VersionsReadmeResourceTest {
    @Mock
    private ConfigurableEnvironment env;
    @Mock
    private VersionNativeResource versionNativeResource;
    @Mock
    private PackageResourceService packageResourceService;
    @Mock
    private NativeSearchServiceFactory nativeSearchServiceFactory;
    @Mock
    private HttpServletRequest servletRequest;
    @Mock
    private HttpHeaders httpHeaders;
    @Mock
    private UriInfo uriInfo;
    @Mock
    private HttpServletResponse servletResponse;
    @Mock
    private Request request;

    private VersionsReadmeResource versionsReadmeResource;

    private AutoCloseable autoCloseable;

    @BeforeEach
    public void setup() {
        autoCloseable = MockitoAnnotations.openMocks(this);
    }

    @AfterEach
    public void cleanup() throws Exception {
        autoCloseable.close();
    }

    @Test
    void expectLegacyReadme() throws PackageException {
        when(env.getActiveProfiles()).thenReturn(new String[] {});
        versionsReadmeResource = new VersionsReadmeResource(nativeSearchServiceFactory, versionNativeResource, env,
                packageResourceService);
        versionsReadmeResource.getReadme(servletRequest, servletResponse, request, uriInfo, httpHeaders);
        Mockito.verify(versionNativeResource)
                .runService(any(), any(HttpServletRequest.class), any(HttpServletResponse.class), any(Request.class),
                        any(UriInfo.class), any(HttpHeaders.class));
    }

    @Test
    void expectLegacyReadmeForOtherProfiles() throws PackageException {
        when(env.getActiveProfiles()).thenReturn(new String[] {"notNpmNew"});
        versionsReadmeResource = new VersionsReadmeResource(nativeSearchServiceFactory, versionNativeResource, env,
                packageResourceService);
        versionsReadmeResource.getReadme(servletRequest, servletResponse, request, uriInfo, httpHeaders);
        Mockito.verify(versionNativeResource)
                .runService(any(), any(HttpServletRequest.class), any(HttpServletResponse.class), any(Request.class),
                        any(UriInfo.class), any(HttpHeaders.class));
    }

    @Test
    void expectNewReadme() throws PackageException {
        MultivaluedHashMap<String, String> queryParams = new MultivaluedHashMap<>();
        queryParams.put("path", List.of("npm-local/colors/-/-colors-1.0.0.tgz"));
        when(env.getActiveProfiles()).thenReturn(new String[] {"npmNew"});
        when(uriInfo.getQueryParameters()).thenReturn(queryParams);
        versionsReadmeResource = new VersionsReadmeResource(nativeSearchServiceFactory, versionNativeResource, env,
                packageResourceService);
        versionsReadmeResource.getReadme(servletRequest, servletResponse, request, uriInfo, httpHeaders);
        Mockito.verify(packageResourceService).getVersionNativeReadme("npm-local/colors/-/-colors-1.0.0.tgz");
    }
}