package org.artifactory.ui.rest.service.admin.configuration.general;

import org.apache.commons.lang3.StringUtils;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.platform.PlatformConfigServiceImpl;
import org.artifactory.platform.PlatformGeneralConfig;
import org.artifactory.security.access.AccessService;
import org.artifactory.service.ConfigValidationService;
import org.artifactory.spring.ArtifactoryApplicationContext;
import org.artifactory.ui.rest.model.admin.configuration.generalconfig.GeneralConfig;
import org.jetbrains.annotations.NotNull;
import org.jfrog.access.client.platformconfig.GrpcPlatformConfigClient;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import java.util.Optional;

import static org.artifactory.ui.rest.service.admin.configuration.general.UpdateConfigUtils.prepareAddonsManager;
import static org.artifactory.ui.rest.service.admin.configuration.general.UpdateConfigUtils.prepareMutableCentralConfigDescriptor;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertTrue;

/**
 * @author Alexei Vainshtein
 */
public class UpdateGeneralConfigServiceTest {

    private UpdateGeneralConfigService updateGeneralConfigService;

    @BeforeClass
    public void beforeClass() {
        MutableCentralConfigDescriptor descriptor = prepareMutableCentralConfigDescriptor();
        ArtifactoryApplicationContext applicationContext = mock(ArtifactoryApplicationContext.class);
        AccessService accessService = mock(AccessService.class);
        GrpcPlatformConfigClient grpcPlatformConfigClient = mock(GrpcPlatformConfigClient.class);
        when(applicationContext.beanForType(AccessService.class)).thenReturn(accessService);
        when(accessService.getPlatformConfigClient()).thenReturn(grpcPlatformConfigClient);
        when(grpcPlatformConfigClient.getConfig(anyString(), eq(PlatformGeneralConfig.class)))
                .thenReturn(Optional.empty());
        CentralConfigService testCentralConfigService = mock(CentralConfigService.class);
        when(testCentralConfigService.getMutableDescriptor()).thenReturn(descriptor);
        updateGeneralConfigService = new UpdateGeneralConfigService(testCentralConfigService, prepareAddonsManager(),
                new ConfigValidationService(), new PlatformConfigServiceImpl(accessService));
        doAnswer(invocationOnMock -> {
            MutableCentralConfigDescriptor mutableCentralConfigDescriptor =
                    (MutableCentralConfigDescriptor) invocationOnMock.getArguments()[0];
            assertTrue(mutableCentralConfigDescriptor.isArchiveIndexEnabled());
            return null;
        }).when(testCentralConfigService).saveEditedDescriptorAndReload(descriptor);
        ArtifactoryContextThreadBinder.bind(applicationContext);
    }

    @AfterClass
    public void afterClass() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testUpdateDescriptorAndSave() {
        updateGeneralConfigService.updateDescriptorAndSave(prepareGeneralConfig("cool"));
    }

    @Test
    public void testUpdateDescriptorAndSaveWithLogoUrl() {
        updateGeneralConfigService.updateDescriptorAndSave(
                prepareGeneralConfig("cool", "https://artifactory.com/logo.png"));
    }

    @Test(expectedExceptions = IllegalArgumentException.class,
            expectedExceptionsMessageRegExp = "Logo URL is not valid")
    public void testUpdateDescriptorAndSaveBadLogoUrl() {
        updateGeneralConfigService.updateDescriptorAndSave(
                prepareGeneralConfig("cool", "https://artifactory.com/1.php"));
    }

    @Test(expectedExceptions = IllegalArgumentException.class,
            expectedExceptionsMessageRegExp = "Illegal name.*is not allowed")
    public void testUpdateDescriptorAndSaveBadServerName() {
        updateGeneralConfigService.updateDescriptorAndSave(prepareGeneralConfig("<a href=http://attacker.com a"));
    }

    @Test
    public void testUpdateDescriptorAndSaveBlankServerName() {
        updateGeneralConfigService.updateDescriptorAndSave(prepareGeneralConfig(StringUtils.EMPTY));
    }

    @NotNull
    public GeneralConfig prepareGeneralConfig(String serverName) {
        return prepareGeneralConfig(serverName, null);
    }

    @NotNull
    public GeneralConfig prepareGeneralConfig(String serverName, String logoUrl) {
        GeneralConfig config = new GeneralConfig();
        config.setServerName(serverName);
        config.setLogoUrl(logoUrl);
        config.setArchiveIndexEnabled(true);
        config.setHelpLinksEnabled(true);
        config.setFileUploadMaxSize(10);
        config.setShowAddonSettings(false);
        config.setGlobalOfflineMode(true);
        config.setSystemMessageEnabled(false);
        config.setFolderDownloadEnabled(false);
        config.setFolderDownloadEnabled(false);
        return config;
    }
}
