package org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.p2.P2Repo;
import org.artifactory.addon.webstart.ArtifactWebstartAddon;
import org.artifactory.aol.AolService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.config.RepositoryConfigService;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.property.ArtifactorySystemProperties;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.repo.ProxyDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.model.FederatedRepoConfig;
import org.artifactory.model.LocalConfig;
import org.artifactory.model.LocalRepoConfig;
import org.artifactory.model.ProxyRepoConfig;
import org.artifactory.model.RemoteRepoConfig;
import org.artifactory.model.VirtualRepoConfig;
import org.artifactory.repo.CdnRedirectRepoConfig;
import org.artifactory.repo.virtual.VirtualRepositoryConfigResolver;
import org.artifactory.spring.InternalArtifactoryContext;
import org.artifactory.ui.rest.model.admin.configuration.repository.RepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteCacheRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CargoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ConanTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DockerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualRepositoryConfigModel;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import java.util.List;
import java.util.Set;

import static org.artifactory.util.PackageTypeConfigFields.FORCE_CONAN_AUTHENTICATION;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertNull;
import static org.testng.Assert.assertTrue;

/**
 * Author: Ivan Perepelytsya
 * Date: 6/2/20.
 */
@Test
public class RepoConfigModelBuilderTest {

    private CentralConfigService centralConfigService;
    private RepoConfigModelBuilder repoConfigModelBuilder;

    @BeforeMethod
    public void bindArtifactoryHome() {
        InternalArtifactoryContext context = mock(InternalArtifactoryContext.class);
        ArtifactoryContextThreadBinder.bind(context);
        AolService aolService = mock(AolService.class);
        when(aolService.isAol()).thenReturn(false);
        when(context.beanForType(AolService.class)).thenReturn(aolService);
        centralConfigService = mock(CentralConfigService.class);
        repoConfigModelBuilder = getRepoConfigModelBuilder();
        when(centralConfigService.getDescriptor()).thenReturn(mock(CentralConfigDescriptor.class));
        final ArtifactorySystemProperties artifactorySystemProperties = mock(ArtifactorySystemProperties.class);
        final ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        when(artifactoryHome.getArtifactoryProperties()).thenReturn(artifactorySystemProperties);
        when(artifactorySystemProperties.getProperty(any())).thenReturn("");
        ArtifactoryHome.bind(artifactoryHome);
    }

    @AfterMethod
    public void unbindArtifactoryHome() {
        ArtifactoryHome.unbind();
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testPopulateLocalDescriptorCdn() {
        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(centralConfigService);

        final LocalConfig localRepoDescriptor = LocalRepoConfig.create("repoKey", RepoType.Generic);
        localRepoDescriptor.getRepoTypeConfig().setCdnRedirectRepoConfig(getEnabledCdnConfig());

        final LocalRepositoryConfigModel localRepositoryConfigModel =
                new LocalRepositoryConfigModel();
        repoConfigModelBuilder.populateLocalDescriptorValuesToModel(localRepoDescriptor,
                localRepositoryConfigModel);

        assertNotNull(localRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig());
        assertTrue(localRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig().isEnabled());
    }

    @Test
    public void testPopulateRemoteDescriptorCdn() {
        final RemoteRepoConfig remoteRepoConfig = RemoteRepoConfig.create("repoKey", RepoType.Generic);
        remoteRepoConfig.getRepoTypeConfig().setCdnRedirectRepoConfig(getEnabledCdnConfig());

        final RemoteRepositoryConfigModel remoteRepositoryConfigModel = new RemoteRepositoryConfigModel();
        repoConfigModelBuilder
                .populateRemoteRepositoryConfigValuesToModel(remoteRepoConfig, remoteRepositoryConfigModel);

        assertNotNull(remoteRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig());
        assertTrue(remoteRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig().isEnabled());
    }

    @Test
    public void testPopulateLocalDescriptorNullCdn() {

        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(centralConfigService);

        final LocalConfig localConfig = LocalRepoConfig.create("repoKey", RepoType.Generic);
        localConfig.getRepoTypeConfig().setCdnRedirectRepoConfig(null);

        final LocalRepositoryConfigModel localRepositoryConfigModel =
                new LocalRepositoryConfigModel();
        repoConfigModelBuilder.populateLocalDescriptorValuesToModel(localConfig,
                localRepositoryConfigModel);
        assertNull(localRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig());
    }


    @Test
    public void testPopulateRemoteDescriptorNullCdn() {
        final RemoteRepoConfig remoteRepoConfig = RemoteRepoConfig.create("repoKey", RepoType.Generic);
        remoteRepoConfig.getRepoTypeConfig().setCdnRedirectRepoConfig(null);

        final RemoteRepositoryConfigModel remoteRepositoryConfigModel = new RemoteRepositoryConfigModel();
        repoConfigModelBuilder
                .populateRemoteRepositoryConfigValuesToModel(remoteRepoConfig, remoteRepositoryConfigModel);

        assertNull(remoteRepositoryConfigModel.getAdvanced().getCdnRedirectRepoConfig());
    }

    @Test(description = "Tests repository stored RetrievalCachePeriod, UI gets this for the Advanced -> Cache section" +
            "and displays this on RetrievalCachePeriod field")
    public void testPopulateRemoteRepositoryDockerConfigValuesToModel() {
        RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        RemoteRepoConfig remoteRepoConfig = RemoteRepoConfig.create("repoKey", RepoType.Generic);
        remoteRepoConfig.getRepoTypeConfig().setRetrievalCachePeriodSecs(500L);

        DockerTypeSpecificConfigModel remoteTypeSpecific = (DockerTypeSpecificConfigModel) repoConfigModelBuilder
                .createRemoteTypeSpecific(RepoType.Docker, remoteRepoConfig);

        assertEquals((long) remoteTypeSpecific.getRetrievalCachePeriodSecs(), 500L);
    }

    public void testPopulateRemoteRepositoryGoConfigValuesToModel() {
        RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        RemoteRepoConfig remoteRepoConfig = RemoteRepoConfig.create("repoKey", RepoType.Generic);
        remoteRepoConfig.getRepoTypeConfig().setMissedRetrievalCachePeriodSecs(300L);

        GoTypeSpecificConfigModel remoteTypeSpecific = (GoTypeSpecificConfigModel) repoConfigModelBuilder
                .createRemoteTypeSpecific(RepoType.Go, remoteRepoConfig);

        assertEquals((long) remoteTypeSpecific.getMissedRetrievalCachePeriodSecs(), 300L);
    }

    @Test
    public void testPopulateRemoteRepositoryConanConfigValuesToModel() {
        RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        RemoteRepoConfig remoteRepoConfig = RemoteRepoConfig.create("repoKey", RepoType.Generic);
        ConanTypeSpecificConfigModel remoteTypeSpecific = (ConanTypeSpecificConfigModel) repoConfigModelBuilder
                .createRemoteTypeSpecific(RepoType.Conan, remoteRepoConfig);
        assertFalse(remoteTypeSpecific.isForceConanAuthentication());
        remoteRepoConfig.getPackageTypeConfig().put(FORCE_CONAN_AUTHENTICATION.getKey(), Boolean.TRUE.toString());
        remoteTypeSpecific = (ConanTypeSpecificConfigModel) repoConfigModelBuilder
                .createRemoteTypeSpecific(RepoType.Conan, remoteRepoConfig);
        assertTrue(remoteTypeSpecific.isForceConanAuthentication());
    }

    @Test(dataProvider = "proxyRepoConfigsAndModels")
    public void testPopulateProxy(ProxyRepoConfig repoConfig, RepositoryConfigModel repositoryConfigModel) {
        populateRepoConfigToModel(repoConfig, repositoryConfigModel);
        // proxy does not exist
        assertNull(getProxyKey(repositoryConfigModel));

        // create default proxy
        ProxyDescriptor defaultProxy = new ProxyDescriptor();
        defaultProxy.setKey("my-proxy");
        defaultProxy.setPlatformDefault(true);
        when(centralConfigService.getDescriptor().getDefaultProxy()).thenReturn(defaultProxy);

        repoConfig.setDisableProxy(true);
        populateRepoConfigToModel(repoConfig, repositoryConfigModel);

        // proxy is disabled
        assertNull(getProxyKey(repositoryConfigModel));

        repoConfig.setDisableProxy(false);
        populateRepoConfigToModel(repoConfig, repositoryConfigModel);

        // proxy should be the default
        assertEquals(getProxyKey(repositoryConfigModel), defaultProxy.getKey());

        String proxyRefKey = "some proxy ref";
        repoConfig.setProxyRef(proxyRefKey);
        when(centralConfigService.getDescriptor().getProxy(proxyRefKey)).thenReturn(new ProxyDescriptor());
        populateRepoConfigToModel(repoConfig, repositoryConfigModel);
        // proxy ref should override the default
        assertEquals(getProxyKey(repositoryConfigModel), proxyRefKey);
    }

    private String getProxyKey(RepositoryConfigModel model) {
        if (model instanceof RemoteRepositoryConfigModel remoteRepositoryConfigModel) {
            return remoteRepositoryConfigModel.getAdvanced().getNetwork().getProxy();
        }
        if (model instanceof FederatedRepositoryConfigModel federatedRepositoryConfigModel) {
            return federatedRepositoryConfigModel.getAdvanced().getProxy();
        }
        Assert.fail(String.format("failed to get proxy key from model '%s'", model.getClass().getSimpleName()));
        return null;
    }

    private void populateRepoConfigToModel(ProxyRepoConfig repoConfig, RepositoryConfigModel model) {
        if (repoConfig instanceof RemoteRepoConfig remoteRepoConfig) {
            repoConfigModelBuilder.populateRemoteRepositoryConfigValuesToModel(remoteRepoConfig,
                    (RemoteRepositoryConfigModel) model);
        } else if (repoConfig instanceof FederatedRepoConfig federatedRepoConfig) {
            repoConfigModelBuilder.populateFederatedDescriptorValuesToModel(federatedRepoConfig,
                    (FederatedRepositoryConfigModel) model);
        } else {
            Assert.fail(String.format("failed to populate repo config '%s' to model '%s'",
                    repoConfig.getClass().getSimpleName(), model.getClass().getSimpleName()));
        }
    }

    @DataProvider
    private Object[][] proxyRepoConfigsAndModels() {
        return new Object[][] {
                {RemoteRepoConfig.create("remote-repo", RepoType.Generic), new RemoteRepositoryConfigModel()},
                {FederatedRepoConfig.create("federated-repo", RepoType.Generic), new FederatedRepositoryConfigModel()}};
    }

    @Test
    public void testResolveLocalP2RepoFromUrl() {
        RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();

        String repoName = "repo";
        String p2RepoUrl = "local://" + repoName;

        RepositoryConfigService repositoryConfigService = mock(RepositoryConfigService.class);
        LocalRepoConfig localRepoConfig = LocalRepoConfig.create(repoName, RepoType.Generic);
        when(repositoryConfigService.getLocalOrFederatedRepoConfigByKey(repoName)).thenReturn(localRepoConfig);
        repoConfigModelBuilder.setRepositoryConfigService(repositoryConfigService);

        P2Repo p2Repo = repoConfigModelBuilder.resolveLocalP2RepoFromUrl(p2RepoUrl);

        assertNotNull(p2Repo);
        assertEquals(p2Repo.getRepoKey(), repoName);
        assertEquals(p2Repo.getRepoUrl(), p2RepoUrl);
    }

    @Test
    public void testCreateCacheConfig() {
        RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        RemoteRepoConfig remoteRepoConfig = RemoteRepoConfig.create("repoKey", RepoType.Generic);
        RemoteCacheRepositoryConfigModel cacheConfig = repoConfigModelBuilder.createCacheConfig(remoteRepoConfig);
        long metadataRetrievalTimeoutSecsCache = cacheConfig.getMetadataRetrievalTimeoutSecs();
        long metadataRetrievalTimeoutSecsDescriptor =
                remoteRepoConfig.getRepoTypeConfig().getMetadataRetrievalTimeoutSecs();
        assertEquals(metadataRetrievalTimeoutSecsCache, metadataRetrievalTimeoutSecsDescriptor);

        remoteRepoConfig.getRepoTypeConfig().setMetadataRetrievalTimeoutSecs(20);
        cacheConfig = repoConfigModelBuilder.createCacheConfig(remoteRepoConfig);
        metadataRetrievalTimeoutSecsCache = cacheConfig.getMetadataRetrievalTimeoutSecs();
        metadataRetrievalTimeoutSecsDescriptor = remoteRepoConfig.getRepoTypeConfig().getMetadataRetrievalTimeoutSecs();
        assertEquals(metadataRetrievalTimeoutSecsCache, metadataRetrievalTimeoutSecsDescriptor);
    }

    @Test
    public void testPopulateVirtualDescriptorCargo() {
        VirtualRepositoryConfigResolver virtualRepositoryConfigResolver = mock(VirtualRepositoryConfigResolver.class);
        when(virtualRepositoryConfigResolver.getOrderedRepos("repoKey")).thenReturn(List.of());
        RepositoryConfigService repositoryConfigService = mock(RepositoryConfigService.class);
        when(repositoryConfigService.findRepositoryConfigsByKeys(any())).thenReturn(Set.of());
        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(centralConfigService);

        final VirtualRepoConfig virtualRepoConfig = VirtualRepoConfig.create("repoKey", RepoType.Cargo);

        final VirtualRepositoryConfigModel virtualRepositoryConfigModel =
                new VirtualRepositoryConfigModel();
        repoConfigModelBuilder.setResolver(virtualRepositoryConfigResolver);
        repoConfigModelBuilder.setRepositoryConfigService(repositoryConfigService);
        repoConfigModelBuilder.populateVirtualRepositoryConfigValuesToModel(virtualRepoConfig,
                virtualRepositoryConfigModel);

        assertTrue(virtualRepositoryConfigModel.getTypeSpecific() instanceof CargoTypeSpecificConfigModel);
        assertFalse(
                ((CargoTypeSpecificConfigModel) virtualRepositoryConfigModel.getTypeSpecific()).isAnonymousAccess());
        assertTrue(((CargoTypeSpecificConfigModel) virtualRepositoryConfigModel.getTypeSpecific()).isInternalIndex());

    }

    private CdnRedirectRepoConfig getEnabledCdnConfig() {
        CdnRedirectRepoConfig cdnRedirectRepoConfig = new CdnRedirectRepoConfig();
        cdnRedirectRepoConfig.setEnabled(true);
        return cdnRedirectRepoConfig;
    }


    private RepoConfigModelBuilder getRepoConfigModelBuilder() {
        final RepoConfigModelBuilder repoConfigModelBuilder = new RepoConfigModelBuilder();
        repoConfigModelBuilder.setCentralConfig(centralConfigService);
        final AddonsManager addonsManager = mock(AddonsManager.class);
        final ArtifactWebstartAddon artifactWebstartAddon = mock(ArtifactWebstartAddon.class);
        when(artifactWebstartAddon.getSslCertNames()).thenReturn(List.of());
        when(addonsManager.addonByType(ArtifactWebstartAddon.class)).thenReturn(artifactWebstartAddon);
        repoConfigModelBuilder.setAddonsManager(addonsManager);
        return repoConfigModelBuilder;
    }
}