package org.artifactory.ui.rest.service.builds.buildsinfo;

import org.artifactory.api.build.BuildService;
import org.artifactory.api.rest.build.ContinueBuildFilter;
import org.artifactory.api.rest.common.model.continues.ContinueResult;
import org.artifactory.common.ConstantValues;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.artifactory.test.ArtifactoryHomeBoundJUnitTest;
import org.artifactory.test.ArtifactoryHomeStub;
import org.artifactory.ui.rest.service.builds.dummies.BuildServiceDummy;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.UriInfo;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.when;

class GetAllBuildNamesPagedServiceTest extends ArtifactoryHomeBoundJUnitTest {

    @Mock
    private UriInfo uriInfo;
    @Mock
    private ArtifactoryRestRequest request;
    private ArtifactoryRestResponse response;
    private AutoCloseable autoCloseable;
    private final ArtifactoryHomeStub artifactoryHomeStub = getOrCreateArtifactoryHomeStub();
    private List<String> buildIds;

    private GetAllBuildNamesPagedService service;

    @BeforeEach
    void setUp() {
        autoCloseable = MockitoAnnotations.openMocks(this);
        artifactoryHomeStub.setProperty(ConstantValues.uiContinuePagingLimit, "5");
        service = new GetAllBuildNamesPagedService();
        BuildService buildService = new BuildServiceDummyInstance();
        service.setBuildService(buildService);
        response = new ArtifactoryRestResponse();
        buildIds = new ArrayList<>();
    }

    private void setMockReturns(String continueState, String limit) {
        MultivaluedMap<String, String> paramsMap = new MultivaluedHashMap<>();
        paramsMap.add("continueState", continueState);
        paramsMap.add("limit", limit);
        when(uriInfo.getQueryParameters()).thenReturn(paramsMap);
        when(request.getUriInfo()).thenReturn(uriInfo);
    }

    @AfterEach
    void tearDown() throws Exception {
        autoCloseable.close();
    }

    @Test
    void testExecuteForAllBuilds() {
        setMockReturns("", "4");
        buildIds.addAll(List.of("C", "A", "D", "B"));

        service.execute(request, response);

        ContinueResult<String> continueResult = (ContinueResult<String>) response.getIModel();
        assertNotNull(continueResult.getContinueState());
        assertEquals("A", continueResult.getData().get(0));
        assertEquals("B", continueResult.getData().get(1));
        assertEquals("C", continueResult.getData().get(2));
        assertEquals("D", continueResult.getData().get(3));
    }

    private class BuildServiceDummyInstance extends BuildServiceDummy {

        @Override
        public List<String> getDistinctBuildNamesPaging(ContinueBuildFilter continueBuildFilter) {
            return buildIds.stream()
                    .sorted()
                    .collect(Collectors.toList());
        }
    }
}