package org.artifactory.ui.rest.service.utils.setMeUp;

import org.artifactory.api.repo.RepositoryService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.descriptor.repo.FederatedRepoDescriptor;
import org.artifactory.descriptor.repo.LocalRepoDescriptor;
import org.artifactory.descriptor.repo.RepoDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;

/**
 * @author Alexei Vainshtein
 */
public class SettingsHelperTest {

    @Test
    public void testGetReadableLocalRepoDescriptors() {
        RepositoryService repositoryService = mock(RepositoryService.class);
        List<LocalRepoDescriptor> reposList = createList();
        when(repositoryService.getLocalAndFederatedRepoDescriptorsExcludingInfoRepos()).thenReturn(reposList);
        AuthorizationService authorizationService = mock(AuthorizationService.class);
        when(authorizationService.canRead(any())).thenReturn(true);
        List<RepoDescriptor> readableLocalRepoDescriptors =
                SettingsHelper.getReadableLocalRepoDescriptors(repositoryService, authorizationService);
        assertEquals(readableLocalRepoDescriptors.size(), reposList.size());
    }

    private List<LocalRepoDescriptor> createList() {
        List<LocalRepoDescriptor> localRepoDescriptors = new ArrayList<>();
        localRepoDescriptors.add(createLocalRepo("key", RepoType.Gradle));
        localRepoDescriptors.add(createLocalRepo("key2", RepoType.Maven));
        localRepoDescriptors.add(createFederatedRepos("key3", RepoType.Ivy));
        localRepoDescriptors.add(createFederatedRepos("key4", RepoType.Gradle));
        return localRepoDescriptors;
    }

    private LocalRepoDescriptor createLocalRepo(String key, RepoType type) {
        LocalRepoDescriptor localRepoDescriptor = new LocalRepoDescriptor();
        localRepoDescriptor.setKey(key);
        localRepoDescriptor.setType(type);
        return localRepoDescriptor;
    }

    private FederatedRepoDescriptor createFederatedRepos(String key, RepoType type) {
        FederatedRepoDescriptor federatedRepo = new FederatedRepoDescriptor();
        federatedRepo.setKey(key);
        federatedRepo.setType(type);
        return federatedRepo;
    }
}