package org.artifactory.rest.resource.aql;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.aql.AqlException;
import org.artifactory.aql.AqlService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.test.util.ReflectionTestUtils;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;

import javax.ws.rs.core.Response;
import java.sql.SQLTimeoutException;
import java.util.stream.Stream;

import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.when;

@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@ExtendWith(MockitoExtension.class)
public class AqlResourceTest {

    @Mock
    private AqlService aqlService;

    @Mock
    private AuthorizationService authorizationService;

    private AutoCloseable autoCloseable;

    @BeforeTest
    public void setup() {
        autoCloseable = MockitoAnnotations.openMocks(this);
        when(authorizationService.isAnonymous()).thenReturn(false);
    }

    @AfterTest
    public void tearDown() throws Exception {
        autoCloseable.close();
    }

    @ParameterizedTest
    @MethodSource("provideExceptions")
    public void testExecuteQueryTimeout(AqlException e) {
        AqlResource aqlResource = new AqlResource();
        ReflectionTestUtils.setField(aqlResource, "authorizationService", authorizationService);
        ReflectionTestUtils.setField(aqlResource, "aqlService", aqlService);
        when(aqlService.executeQueryLazy(anyString())).thenThrow(e);
        Assertions.assertEquals(aqlResource.execute(false, "something").getStatus(),
                Response.Status.REQUEST_TIMEOUT.getStatusCode());
    }

    private static Stream<Arguments> provideExceptions() {
        return Stream.of(
                Arguments.of(new AqlException("Failed to execute the following sql query: something",
                        new SQLTimeoutException("The statement has been cancelled or timed out."))),// derby
                Arguments.of(new AqlException("Failed to execute the following sql query: something",
                        new Exception("ERROR: canceling statement due to user request"))), //psql
                Arguments.of(new AqlException("Failed to execute the following sql query: something",
                        new SQLTimeoutException("The query has timed out."))), //mssql
                Arguments.of(new AqlException("Failed to execute the following sql query: something",
                        new SQLTimeoutException("Statement cancelled due to timeout or client request"))) //mysql
        );
    }
}